@@ -91,13 +91,18 @@ def remote_openai_server(request):
             "Error setting up remote_openai_server params") from e
 
     if 'cb' in params:
-        max_num_seqs = params['max_num_seqs']
+        max_model_len = params["max_model_len"]
+        max_num_seqs = params["max_num_seqs"]
         env_dict = {
             "VLLM_SPYRE_USE_CB": "1",
             "VLLM_SPYRE_DYNAMO_BACKEND": backend,
             "VLLM_USE_V1": "1"
         }
-        server_args = ["--max_num_seqs", str(max_num_seqs)]
+        server_args = [
+            "--max_num_seqs",
+            str(max_num_seqs), "--max-model-len",
+            str(max_model_len)
+        ]
 
     else:
         warmup_shape = params['warmup_shape']
@@ -126,7 +131,7 @@ def remote_openai_server(request):
         if 'tp_size' in params:
             tp_size = params['tp_size']
             skip_unsupported_tp_size(int(tp_size), backend)
-        server_args.extend(["--tensor-parallel-size", str(tp_size)])
+            server_args.extend(["--tensor-parallel-size", str(tp_size)])
 
     try:
         with RemoteOpenAIServer(model, server_args,
